 /*
  * Copyright (C), 2018-2020
  * Project: starter
  * FileName: AssignServiceHandler
  * Author:   
  * Date:     2020/9/11 8:43
  */
 package com.fr.plugin.cdia.service;

 import com.fanruan.api.decision.user.UserKit;
 import com.fanruan.api.i18n.I18nKit;
 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.net.http.HttpKit;
 import com.fanruan.api.util.StringKit;
 import com.fr.decision.authority.AuthorityContext;
 import com.fr.decision.authority.data.CustomRole;
 import com.fr.decision.fun.impl.BaseHttpHandler;
 import com.fr.decision.webservice.bean.user.UserBean;
 import com.fr.decision.webservice.v10.user.UserService;
 import com.fr.general.ComparatorUtils;
 import com.fr.json.JSONArray;
 import com.fr.json.JSONException;
 import com.fr.json.JSONObject;
 import com.fr.plugin.cdia.ProcessDBAccess;
 import com.fr.plugin.cdia.config.CdiaConfig;
 import com.fr.plugin.cdia.dao.ProcessAssignDao;
 import com.fr.plugin.cdia.entity.ProcessAssignEntity;
 import com.fr.plugin.cdia.submit.job.AddUserSubmitJob;
 import com.fr.plugin.cdia.submit.job.CustomSubmitJob;
 import com.fr.plugin.context.PluginContexts;
 import com.fr.stable.StringUtils;
 import com.fr.stable.db.action.DBAction;
 import com.fr.stable.db.dao.DAOContext;
 import com.fr.stable.query.QueryFactory;
 import com.fr.stable.query.condition.QueryCondition;
 import com.fr.stable.query.restriction.RestrictionFactory;
 import com.fr.third.org.apache.http.entity.StringEntity;
 import com.fr.third.springframework.web.bind.annotation.RequestMethod;
 import com.fr.web.utils.WebUtils;

 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;

 /**
  * <Function Description><br>
  * <AssignServiceHandler>
  *
  * @author 
  * @since 1.0.0
  */
 public class AssignServiceHandler extends BaseHttpHandler {

     public static final String API_ASSIGN_PROCESS = "/cdia/assignProcess";
     public static final String TC_OP_YES = "同意";
     public static final String AUTH_GROUP = "performer";
     public static final String USER_WAIT_TO_ASSIGN_NMA_AUTH_LST_HEADER = "<userWaitToAssignNmaAuthLst><userWaitToAssignNmaAuthMap>";
     public static final String USER_WAIT_TO_ASSIGN_NMA_AUTH_LST_FOOTER = "</userWaitToAssignNmaAuthMap></userWaitToAssignNmaAuthLst>";
     public static final String SOAP_REQUEST_HEADER = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:aut=\"http://services.tesla.com/autoAssignRoleAndDataAuthAndMdReportAuthService\"><soapenv:Header/><soapenv:Body><aut:autoAssignRoleAndDataAuthAndMdReportAuthServiceRequest>";
     public static final String SOAP_SERVICE_FOOTER = "</aut:autoAssignRoleAndDataAuthAndMdReportAuthServiceRequest></soapenv:Body></soapenv:Envelope>";
     public static final String YT_OP_YES = "[同意]";

     public AssignServiceHandler() {
     }

     @Override
     public RequestMethod getMethod() {
         return RequestMethod.POST;
     }

     @Override
     public String getPath() {
         return API_ASSIGN_PROCESS;
     }

     @Override
     public boolean isPublic() {
         return true;
     }

     @Override
     public void handle(HttpServletRequest request, HttpServletResponse response) throws Exception {
         if (PluginContexts.currentContext() == null || !PluginContexts.currentContext().isAvailable()) {
             LogKit.error(I18nKit.getLocText("Plugin-cdia_Licence_Expired"));
             WebUtils.flushSuccessMessageAutoClose(request, response, createFailResponseJSONObject());
             return;
         }
         try {
             JSONObject paramsJSON = getPostParams(request);
             String procInstId = paramsJSON.getString("procInstId", StringUtils.EMPTY);
             String msgType = paramsJSON.getString("msgType", StringUtils.EMPTY);
             String procEndFlag = "0";
             String procDeleteFlag = "0";
             if (ComparatorUtils.equals(msgType, "delete") || ComparatorUtils.equals(msgType, "break")) {
                 procDeleteFlag = "1";
             } else if (ComparatorUtils.equals(msgType, "commit")) {
                 procEndFlag = "1";
             }
             List<ProcessAssignEntity> processAssignEntityList = ProcessAssignDao.findEntityByProc(procInstId);
             StringBuilder userWaitToAssignNmaAuthLst = new StringBuilder();
             for (ProcessAssignEntity processAssignEntity : processAssignEntityList) {
                 processAssignEntity.setProcEndFlag(Integer.parseInt(procEndFlag));
                 processAssignEntity.setProcDeleteFlag(Integer.parseInt(procDeleteFlag));

                 setProcNodeOpinion(processAssignEntity);
                 updateEntity(processAssignEntity);

                 LogKit.info("cdia-AssignServiceHandler-handle-头寸审批意见:{}", processAssignEntity.getTcOp());
                 if (ComparatorUtils.equals(processAssignEntity.getTcOp(), TC_OP_YES)) {
                     LogKit.info("cdia-AssignServiceHandler-handle-申请人用户ID:{}", processAssignEntity.getAppId());
                     LogKit.info("cdia-AssignServiceHandler-handle-头寸权限:{}", processAssignEntity.getTcRoleName());
                     if (!setUserRole(processAssignEntity.getAppId(), processAssignEntity.getTcRoleName())) {
                         LogKit.info("cdia-AssignServiceHandler-handle-更新头寸权限失败！");
                     }
                 }
                 LogKit.info("cdia-AssignServiceHandler-handle-YT_OP审批意见:{}", processAssignEntity.getYtOp());
                 if (processAssignEntity.getYtOp().startsWith(YT_OP_YES)) {
                     // 产生管会接口数据
                     assignNmaAuthLst(userWaitToAssignNmaAuthLst, processAssignEntity);
                 }
             }
             if (StringKit.isNotBlank(userWaitToAssignNmaAuthLst.toString())) {
                 sendAssignNmaAuthLst(userWaitToAssignNmaAuthLst);
             }
             WebUtils.flushSuccessMessageAutoClose(request, response, createSuccessResponseJSONObject());
         } catch (Exception e) {
             WebUtils.flushSuccessMessageAutoClose(request, response, createFailResponseJSONObject());
             LogKit.error(e.getMessage(), e);
         }
     }

     /**
      * 发送管会接口
      *
      * @param userWaitToAssignNmaAuthLst
      * @throws IOException
      */
     private void sendAssignNmaAuthLst(StringBuilder userWaitToAssignNmaAuthLst) throws IOException {
         Map<String, String> headers = new HashMap<>();
         headers.put("Content-Type", "text/xml;charset=UTF-8");
         headers.put("SOAPAction", "autoAssignRoleAndDataAuthAndMdReportAuthService");
         String soapXml = SOAP_REQUEST_HEADER +
                 userWaitToAssignNmaAuthLst.toString() +
                 SOAP_SERVICE_FOOTER;
         LogKit.info("cdia-AssignServiceHandler-sendAssignNmaAuthLst-soapXml:{}, headers:{}", soapXml, headers);
         StringEntity stringEntity = new StringEntity(soapXml, "UTF-8");
         String response = HttpKit.executeAndParse(com.fanruan.api.net.http.rs.HttpRequest.custom().url(CdiaConfig.getInstance().getAssignNmaAuthUrl()).post(stringEntity).headers(headers).build());
         LogKit.info("cdia-AssignServiceHandler-sendAssignNmaAuthLst-response:{}", response);
     }

     /**
      * 产生管会接口数据
      *
      * @param userWaitToAssignNmaAuthLst
      * @param processAssignEntity
      */
     private void assignNmaAuthLst(StringBuilder userWaitToAssignNmaAuthLst, ProcessAssignEntity processAssignEntity) {
         userWaitToAssignNmaAuthLst.append(USER_WAIT_TO_ASSIGN_NMA_AUTH_LST_HEADER);
         userWaitToAssignNmaAuthLst.append("<empNo>").append(processAssignEntity.getAppNumber()).append("</empNo>");
         userWaitToAssignNmaAuthLst.append("<userNm>").append(processAssignEntity.getAppName()).append("</userNm>");
         userWaitToAssignNmaAuthLst.append("<oaUserNm>").append(processAssignEntity.getAppId()).append("</oaUserNm>");
         userWaitToAssignNmaAuthLst.append("<instNm>").append(processAssignEntity.getAppOperName()).append("</instNm>");
         userWaitToAssignNmaAuthLst.append("<deptNm>").append(processAssignEntity.getAppDeptName()).append("</deptNm>");
         userWaitToAssignNmaAuthLst.append("<mobile>").append(processAssignEntity.getAppTel()).append("</mobile>");
         userWaitToAssignNmaAuthLst.append("<email>").append(processAssignEntity.getAppMail()).append("</email>");
         userWaitToAssignNmaAuthLst.append("<userType>").append(processAssignEntity.getAppType()).append("</userType>");
         userWaitToAssignNmaAuthLst.append("<purpose>").append(processAssignEntity.getPurpose()).append("</purpose>");
         userWaitToAssignNmaAuthLst.append("<authType>").append(processAssignEntity.getBtRoleName()).append("</authType>");
         userWaitToAssignNmaAuthLst.append(USER_WAIT_TO_ASSIGN_NMA_AUTH_LST_FOOTER);
     }

     /**
      * 获得提交接口的JSON参数
      *
      * @param request
      * @return
      */
     private JSONObject getPostParams(HttpServletRequest request) {
         BufferedReader reader = null;
         StringBuilder sb = new StringBuilder();
         try {
             reader = new BufferedReader(new InputStreamReader(request.getInputStream(), "utf-8"));
             String line = null;
             while ((line = reader.readLine()) != null) {
                 sb.append(line);
             }
         } catch (IOException e) {
             LogKit.error(e.getMessage(), e);
         } finally {
             try {
                 if (null != reader) {
                     reader.close();
                 }
             } catch (IOException e) {
                 LogKit.error(e.getMessage(), e);
             }
         }
         if (StringUtils.isEmpty(sb.toString())) {
             return JSONObject.EMPTY;
         }
         LogKit.info("cdia-AssignServiceHandler-getPostParams:{}", sb.toString());
         return new JSONObject(sb.toString());
     }

     /**
      * 审批意见查询接口，返回审核意见
      *
      * @param processAssignEntity
      */
     private void setProcNodeOpinion(ProcessAssignEntity processAssignEntity) throws Exception {
         JSONObject bodyJSON = JSONObject.create();
         JSONObject paramObject = JSONObject.create();
         paramObject.put("procInstId", processAssignEntity.getProcInstId());
         paramObject.put("authGroup", AUTH_GROUP);
         paramObject.put("nodeInstId", StringUtils.EMPTY);
         bodyJSON.put("paramObject", paramObject);
         bodyJSON.put("sysId", CustomSubmitJob.SYS_ID);
         bodyJSON.put("userId", processAssignEntity.getDrafId());
         LogKit.info("cdia-AssignServiceHandler-setProcNodeOpinion-审批意见查询接口参数:{}", bodyJSON.encode());
         String res = CustomSubmitJob.postJSON(CdiaConfig.getInstance().getProcNodeUrl(), bodyJSON);
         LogKit.info("cdia-AssignServiceHandler-审批意见查询接口响应:{}", res);
         JSONArray opinionMap = (new JSONObject(res)).getJSONObject("resultData").getJSONArray("opinionMap");
         String ytOp = StringUtils.EMPTY;
         String tcOp = StringUtils.EMPTY;
         for (int i = 0; i < opinionMap.size(); i++) {
             String fieldCode = opinionMap.getJSONObject(i).getString("fieldCode");
             String opinionType = opinionMap.getJSONObject(i).getString("opinionText");
             if (ComparatorUtils.equals(fieldCode, "cloudAtlasOption")) {
                 // 价值云图权限审批意见
                 ytOp = opinionType;
             }
             if (ComparatorUtils.equals(fieldCode, "positionOption")) {
                 // 头寸权限审批意见
                 tcOp = opinionType;
             }
         }
         processAssignEntity.setYtOp(ytOp);
         processAssignEntity.setTcOp(tcOp);
     }

     /**
      * 保存
      *
      * @param processAssignEntity
      * @throws Exception
      */
     private void updateEntity(final ProcessAssignEntity processAssignEntity) throws Exception {
         ProcessDBAccess.getAccessor().runDMLAction(new DBAction<ProcessAssignEntity>() {
             @Override
             public ProcessAssignEntity run(DAOContext daoContext) throws Exception {
                 (daoContext.getDAO(ProcessAssignDao.class)).update(processAssignEntity);
                 return null;
             }
         });
     }

     /**
      * @param userName
      * @param tcRoleName
      * @return
      * @throws Exception
      */
     private Boolean setUserRole(String userName, final String tcRoleName) throws Exception {
         if (StringUtils.isEmpty(tcRoleName) || StringUtils.isEmpty(userName)) {
             return false;
         }
         if (UserKit.existUsername(userName)) {
             try {
                 String roleId = getRoleIDByName(tcRoleName);
                 String userId = UserService.getInstance().getCurrentUserId(userName);
                 UserBean userBean = new UserBean();
                 userBean.setId(userId);
                 userBean.setRoleIds(new String[]{roleId});
                 UserService.getInstance().updateUserRoles(AddUserSubmitJob.getAdminUserId(), userBean);
                 return true;
             } catch (Exception e) {
                 LogKit.error(e.getMessage(), e);
             }
         }
         return false;
     }

     private String getRoleIDByName(String role) throws Exception {
         QueryCondition condition = QueryFactory.create().addRestriction(RestrictionFactory.eq("name", role));
         CustomRole customRole = AuthorityContext.getInstance().getCustomRoleController().findOne(condition);
         if (customRole != null) {
             return customRole.getId();
         }
         return StringKit.EMPTY;
     }

     private JSONObject createSuccessResponseJSONObject() throws JSONException {
         JSONObject result = JSONObject.create();
         result.put("errorCode", 0);
         result.put("status", "success");
         return result;
     }

     private JSONObject createFailResponseJSONObject() throws JSONException {
         JSONObject result = JSONObject.create();
         result.put("errorCode", 1);
         result.put("status", "fail");
         return result;
     }
 }