 /*
  * Copyright (C), 2018-2020
  * Project: starter
  * FileName: QueryServiceHandler
  * Author:   
  * Date:     2020/9/11 8:51
  */
 package com.fr.plugin.cdia.service;

 import com.fanruan.api.i18n.I18nKit;
 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.net.NetworkKit;
 import com.fr.decision.fun.impl.BaseHttpHandler;
 import com.fr.json.JSONArray;
 import com.fr.json.JSONException;
 import com.fr.json.JSONObject;
 import com.fr.plugin.cdia.dao.ProcessAssignDao;
 import com.fr.plugin.cdia.entity.ProcessAssignEntity;
 import com.fr.plugin.context.PluginContexts;
 import com.fr.third.springframework.web.bind.annotation.RequestMethod;
 import com.fr.web.utils.WebUtils;

 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import java.util.List;

 /**
  * <Function Description><br>
  * <QueryServiceHandler>
  *
  * @author 
  * @since 1.0.0
  */
 public class QueryServiceHandler extends BaseHttpHandler {

     public static final String API_QUERY_PROCESS = "/cdia/queryProcess";

     public QueryServiceHandler() {
     }

     @Override
     public RequestMethod getMethod() {
         return RequestMethod.GET;
     }

     @Override
     public String getPath() {
         return API_QUERY_PROCESS;
     }

     @Override
     public boolean isPublic() {
         return true;
     }

     @Override
     public void handle(HttpServletRequest request, HttpServletResponse response) throws Exception {
         if (PluginContexts.currentContext() == null || !PluginContexts.currentContext().isAvailable()) {
             LogKit.error(I18nKit.getLocText("Plugin-cdia_Licence_Expired"));
             WebUtils.flushSuccessMessageAutoClose(request, response, createFailResponseJSONObject());
             return;
         }
         final String procInstId = NetworkKit.getHTTPRequestParameter(request, "procInstId");
         JSONObject data = null;
         try {
             data = getProcessData(procInstId);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
         }
         JSONObject result = createFailResponseJSONObject();
         if (data != null && data.size() > 0) {
             result = createSuccessResponseJSONObject();
             result.put("data", data);
         }
         WebUtils.flushSuccessMessageAutoClose(request, response, result);
     }

     private JSONObject getProcessData(String procInstId) throws Exception {
         List<ProcessAssignEntity> processAssignEntityList = ProcessAssignDao.findEntityByProc(procInstId);
         JSONObject processData = JSONObject.create();
         JSONArray detail = JSONArray.create();
         for (ProcessAssignEntity processAssignEntity : processAssignEntityList) {
             if (!processData.has("header")) {
                 processData.put("header", processAssignEntity.toJSON());
             }
             detail.add(processAssignEntity.detail2JSON());
         }
         processData.put("body", detail);
         return processData;
     }

     private JSONObject createSuccessResponseJSONObject() throws JSONException {
         JSONObject result = JSONObject.create();
         result.put("errorCode", 0);
         result.put("status", "success");
         return result;
     }

     private JSONObject createFailResponseJSONObject() throws JSONException {
         JSONObject result = JSONObject.create();
         result.put("errorCode", 1);
         result.put("status", "fail");
         return result;
     }
 }