 /*
  * Copyright (C), 2018-2020
  * Project: starter
  * FileName: AddUserSubmitJob
  * Author:   
  * Date:     2020/8/30 15:49
  */
 package com.fr.plugin.cdia.submit.job;

 import com.fanruan.api.decision.user.UserKit;
 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.util.StringKit;
 import com.fr.data.DefinedSubmitJob;
 import com.fr.data.JobValue;
 import com.fr.decision.authority.AuthorityContext;
 import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
 import com.fr.decision.authority.data.CustomRole;
 import com.fr.decision.authority.data.Department;
 import com.fr.decision.authority.data.Post;
 import com.fr.decision.privilege.TransmissionTool;
 import com.fr.decision.webservice.bean.user.DepartmentPostBean;
 import com.fr.decision.webservice.bean.user.RoleBean;
 import com.fr.decision.webservice.bean.user.UserBean;
 import com.fr.decision.webservice.v10.user.CustomRoleService;
 import com.fr.decision.webservice.v10.user.PositionService;
 import com.fr.decision.webservice.v10.user.UserService;
 import com.fr.script.Calculator;
 import com.fr.stable.Primitive;
 import com.fr.stable.StringUtils;
 import com.fr.stable.core.UUID;
 import com.fr.stable.query.QueryFactory;
 import com.fr.stable.query.condition.QueryCondition;
 import com.fr.stable.query.restriction.RestrictionFactory;

 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;

 /**
  * <Function Description><br>
  * <自定义提交添加用户AddUserSubmitJob>
  *
  * @author 
  * @since 1.0.0
  */
 public class AddUserSubmitJob extends DefinedSubmitJob {

     public static final String SEPARATOR = ",";

     /**
      * 获取管理员id
      *
      * @return
      * @throws Exception
      */
     public static String getAdminUserId() throws Exception {
         List<String> adminUserIdList = UserService.getInstance().getAdminUserIdList();
         if (adminUserIdList.isEmpty()) {
             return "admin";
         }
         return StringKit.isNotBlank(adminUserIdList.get(0)) ? adminUserIdList.get(0) : "admin";
     }

     private String roleOperation(String role) throws Exception {
         String customRoleId = getCustomRoleIdByName(role);
         if (StringKit.isNotBlank(customRoleId)) {
             return customRoleId;
         }
         RoleBean roleBean = new RoleBean(role, "", role, ManualOperationType.KEY.toInteger());
         CustomRoleService.getInstance().addCustomRole(getAdminUserId(), roleBean);
         return getCustomRoleIdByName(role);
     }

     private String getCustomRoleIdByName(String roleName) throws Exception {
         QueryCondition condition = QueryFactory.create().addRestriction(RestrictionFactory.eq("name", roleName));
         CustomRole customRole = AuthorityContext.getInstance().getCustomRoleController().findOne(condition);
         if (customRole != null) {
             return customRole.getId();
         }
         return StringKit.EMPTY;
     }

     @Override
     public String getJobType() {
         return "AddUserSubmitJob";
     }

     @Override
     public void doJob(Calculator calculator) throws Exception {
         Map<String, Object> propertyMap = calculator.getAttribute(PROPERTY_VALUE);
         if (propertyMap == null) {
             return;
         }
         if (isOperationState(propertyMap)) {
             String departmentName = getAttributeByKey(propertyMap, "department");
             String roles = getAttributeByKey(propertyMap, "role");
             String deptType = getAttributeByKey(propertyMap, "deptType");
             String rolesStr = StringKit.EMPTY;
             if (StringKit.isNotEmpty(roles)) {
                 rolesStr = roles;
             }
             if (StringKit.isNotEmpty(deptType)) {
                 if (StringKit.isNotEmpty(rolesStr)) {
                     rolesStr = rolesStr + "," + deptType;
                 } else {
                     rolesStr = deptType;
                 }
             }
             LogKit.info("cdia-AddUserSubmitJob-doJob-rolesStr", rolesStr);
             UserBean userBean = getUserBean(propertyMap);
             String departmentPostId = getDepartmentPostId(departmentName);
             operation(userBean, rolesStr, departmentPostId);
         }
     }

     /**
      * 判断是否提交填报操作
      *
      * @param propertyMap
      * @return
      */
     private Boolean isOperationState(Map<String, Object> propertyMap) {
         JobValue ce = (JobValue) propertyMap.get("username");
         if (ce.getState() == JobValue.State.MODIFIED) {
             // 此单元格的值在报表初始化后被修改过
             LogKit.info("cdia-AddUserSubmitJob-doJob-State-MODIFIED");
             return true;
         } else if (ce.getState() == JobValue.State.INSERT) {
             // 此单元格是在报表初始化后新增的(例如执行了插入行操作)
             LogKit.info("cdia-AddUserSubmitJob-doJob-State-INSERT");
             return true;
         } else if (ce.getState() == JobValue.State.DELETED) {
             // 此单元格所在的记录被执行了删除操作
         } else if (ce.getState() == JobValue.State.DEFAULT) {
             // 此单元格在报表初始化后没有变化
             LogKit.info("cdia-AddUserSubmitJob-doJob-State-DEFAULT");
             return true;
         }
         return false;
     }

     /**
      * 填报属性提取用户信息
      *
      * @param propertyMap
      * @return
      */
     private UserBean getUserBean(Map<String, Object> propertyMap) {
         String username = getAttributeByKey(propertyMap, "username");
         String name = getAttributeByKey(propertyMap, "name");
         String mobile = getAttributeByKey(propertyMap, "mobile");
         String email = getAttributeByKey(propertyMap, "email");
         UserBean userBean = new UserBean();
         userBean.setUsername(username);
         userBean.setPassword(TransmissionTool.defaultEncrypt(UUID.randomUUID().toString()));
         userBean.setRealName(name);
         userBean.setMobile(mobile);
         userBean.setEmail(email);
         return userBean;
     }

     /**
      * 通过属性名获得填报属性值
      *
      * @param propertyMap
      * @param key
      * @return
      */
     private String getAttributeByKey(Map<String, Object> propertyMap, String key) {
         if (!propertyMap.containsKey(key)) {
             return StringUtils.EMPTY;
         }
         Object value = ((JobValue) propertyMap.get(key)).getValue();
         if (value instanceof Primitive) {
             return StringUtils.EMPTY;
         }
         return String.valueOf(((JobValue) propertyMap.get(key)).getValue());
     }

     /**
      * 更新用户信息和角色
      *
      * @param userBean
      * @param roles
      * @throws Exception
      */
     private void operation(UserBean userBean, final String roles, String departmentPostId) throws Exception {
         if (UserKit.existUsername(userBean.getUsername())) {
             return;
         }
         if (StringKit.isNotBlank(roles)) {
             List<String> roleIds = new ArrayList<>();
             String roleId;
             for (String role : roles.split(SEPARATOR)) {
                 if (StringKit.isEmpty(role)) {
                     continue;
                 }
                 roleId = roleOperation(role);
                 roleIds.add(roleId);
             }
             userBean.setRoleIds(roleIds.toArray(new String[0]));
         }
         if (StringKit.isNotBlank(departmentPostId)) {
             userBean.setDepartmentPostId(departmentPostId);
         }
         UserService.getInstance().addUser(userBean);
     }

     private String getDepartmentPostId(String departmentAndPostName) throws Exception {
         String departmentName;
         String positionName;
         if (departmentAndPostName.contains("-")) {
             String[] departmentAndPostArray = departmentAndPostName.split("-");
             departmentName = departmentAndPostArray[0];
             positionName = departmentAndPostArray[1];
         } else {
             departmentName = departmentAndPostName;
             positionName = StringKit.EMPTY;
         }
         AuthorityContext authorityContext = AuthorityContext.getInstance();
         QueryCondition queryDepartment = QueryFactory.create().addRestriction(RestrictionFactory.eq(Department.COLUMN_NAME, departmentName));
         Department department = authorityContext.getDepartmentController().findOne(queryDepartment);
         if (department == null) {
             return StringKit.EMPTY;
         }
         String departmentPostId = department.getId();
         String positionId = positionSynOperation(positionName, departmentPostId);
         if (StringKit.isNotBlank(positionId)) {
             departmentPostId = departmentPostId + "@@@" + positionId;
         }
         return departmentPostId;
     }

     /**
      * 职务同步操作
      *
      * @param positionName
      * @return
      * @throws Exception
      */
     private String positionSynOperation(String positionName, String departmentId) throws Exception {
         if (StringKit.isBlank(positionName)) {
             return StringKit.EMPTY;
         }
         String position = StringKit.isNotBlank(positionName) ? positionName : "职员";
         Post post = AuthorityContext.getInstance().getPostController().findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("name", position)));
         String positionId;
         if (post == null) {
             positionId = PositionService.getInstance().addPosition(position, position);
         } else {
             positionId = post.getId();
         }
         List<DepartmentPostBean> departmentPostBeanList = PositionService.getInstance().getPositionsUnderParentDepartment(getAdminUserId(), departmentId, position);
         if (departmentPostBeanList == null || departmentPostBeanList.isEmpty()) {
             try {
                 AuthorityContext.getInstance().getPostController().addPostToDepartment(positionId, departmentId);
             } catch (Exception e) {
                 LogKit.info("cdia-AddUserSubmitJob-positionSynOperation-addPostToDepartmentFailed-position:{}, departmentId:{}", positionId + position, departmentId);
                 LogKit.error(e.getMessage(), e);
             }
         }
         return positionId;
     }
 }