 /*
  * Copyright (C), 2018-2020
  * Project: starter
  * FileName: CustomSubmitJob
  * Author:   
  * Date:     2020/8/28 17:01
  */
 package com.fr.plugin.cdia.submit.job;

 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.net.http.HttpKit;
 import com.fr.data.DefinedSubmitJob;
 import com.fr.data.JobValue;
 import com.fr.general.ComparatorUtils;
 import com.fr.json.JSONObject;
 import com.fr.plugin.cdia.ProcessDBAccess;
 import com.fr.plugin.cdia.config.CdiaConfig;
 import com.fr.plugin.cdia.dao.ProcessAssignDao;
 import com.fr.plugin.cdia.entity.ProcessAssignEntity;
 import com.fr.script.Calculator;
 import com.fr.stable.Primitive;
 import com.fr.stable.StringUtils;
 import com.fr.stable.db.action.DBAction;
 import com.fr.stable.db.dao.DAOContext;
 import com.fr.third.org.apache.http.entity.StringEntity;

 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.Map.Entry;

 /**
  * <Function Description><br>
  * <CustomSubmitJob>
  *
  * @author 
  * @since 1.0.0
  */
 public class CustomSubmitJob extends DefinedSubmitJob {

     public static final String SYS_ID = "OA";
     private static final HashSet<String> paramsSet = new HashSet<String>() {{
         add("procDefId");
         add("docTitle");
         add("userId");
         add("deptId");
         add("systemCode");
         add("equipmentType");
     }};

     /**
      * POST提交JSON
      *
      * @param url
      * @param params
      * @return
      * @throws Exception
      */
     public static String postJSON(String url, JSONObject params) throws Exception {
         Map<String, String> headers = new HashMap<>();
         headers.put("Content-Type", "application/json");
         StringEntity stringEntity = new StringEntity(params.encode(), "UTF-8");
         return HttpKit.executeAndParse(com.fanruan.api.net.http.rs.HttpRequest.custom()
                 .url(url).post(stringEntity).headers(headers).build());
     }

     @Override
     public String getJobType() {
         return "CustomJob";
     }

     @Override
     public void doJob(Calculator calculator) throws Exception {
         Map<String, Object> propertyMap = calculator.getAttribute(PROPERTY_VALUE);
         if (propertyMap == null) {
             return;
         }
         if (!isOperationState(propertyMap)) {
             return;
         }
         ProcessAssignEntity processAssignEntity = new ProcessAssignEntity();
         JSONObject paramsJSON = JSONObject.create();
         getSubmitAttribute(propertyMap, processAssignEntity, paramsJSON);
         if (paramsJSON.isEmpty()) {
             return;
         }
         operation(paramsJSON, processAssignEntity);
     }

     /**
      * 提取填报属性值
      *
      * @param propertyMap         属性值Map
      * @param processAssignEntity 流程实体
      * @param paramsJSON          http接口参数
      */
     private void getSubmitAttribute(Map<String, Object> propertyMap, ProcessAssignEntity processAssignEntity, JSONObject paramsJSON) {
         for (Entry<String, Object> entry : propertyMap.entrySet()) {
             if (entry.getValue() instanceof JobValue) {
                 JobValue ce = (JobValue) entry.getValue();
                 if (ce.getValue() instanceof Primitive) {
                     continue;
                 }
                 if (paramsSet.contains(entry.getKey())) {
                     paramsJSON.put(entry.getKey(), String.valueOf(ce.getValue()));
                 }
                 processAssignEntity.setEntity(entry.getKey(), String.valueOf(ce.getValue()));
             }
         }
     }

     /**
      * 填报属性状态判断
      *
      * @param propertyMap
      * @return
      */
     private Boolean isOperationState(Map<String, Object> propertyMap) {
         for (Entry<String, Object> entry : propertyMap.entrySet()) {
             if (entry.getValue() instanceof JobValue) {
                 JobValue ce = (JobValue) entry.getValue();
                 if (ce.getState() == JobValue.State.MODIFIED) {
                     // 此单元格的值在报表初始化后被修改过
                     LogKit.info("cdia-CustomSubmitJob-doJob-State-MODIFIED");
                     return true;
                 } else if (ce.getState() == JobValue.State.INSERT) {
                     // 此单元格是在报表初始化后新增的(例如执行了插入行操作)
                     LogKit.info("cdia-CustomSubmitJob-doJob-State-INSERT");
                     return true;
                 } else if (ce.getState() == JobValue.State.DEFAULT) {
                     // 此单元格在报表初始化后没有变化
                     LogKit.info("cdia-CustomSubmitJob-doJob-State-DEFAULT");
                     return true;
                 }
             }
             return false;
         }
         return false;
     }

     /**
      * 保存
      *
      * @param processAssignEntity
      * @throws Exception
      */
     private void save(final ProcessAssignEntity processAssignEntity) throws Exception {
         ProcessDBAccess.getAccessor().runDMLAction(new DBAction<ProcessAssignEntity>() {
             @Override
             public ProcessAssignEntity run(DAOContext daoContext) throws Exception {
                 (daoContext.getDAO(ProcessAssignDao.class)).addOrUpdate(processAssignEntity);
                 return null;
             }
         });
     }

     /**
      * 提交属性处理操作
      * 1.访问创建流程接口，获得流程ID及其他信息
      * 2.流程表单保存
      * 3.打开浏览器窗口
      *
      * @param paramsJSON
      * @param processAssignEntity
      * @throws Exception
      */
     private void operation(JSONObject paramsJSON, ProcessAssignEntity processAssignEntity) throws Exception {
         JSONObject createProcessBody = saveProcessBody(paramsJSON);
         LogKit.info("cdia-CustomSubmitJob-operation-createProcessBody:{}", createProcessBody.encode());
         String res = postJSON(CdiaConfig.getInstance().getCreateProcessUrl(), createProcessBody);
         LogKit.info("cdia-CustomSubmitJob-operation-response:{}", res);
         JSONObject procInstanceJo = new JSONObject(res).getJSONObject("resultData").getJSONObject("procInstance");
         String procInstId = procInstanceJo.getString("procInstId");
         String sequenceNo = procInstanceJo.getString("sequenceNo");
         if (StringUtils.isEmpty(procInstId) || ComparatorUtils.equals(procInstId, "{}")
                 || StringUtils.isEmpty(sequenceNo) || ComparatorUtils.equals(sequenceNo, "{}")
         ) {
             return;
         }
         // 保存数据库
         processAssignEntity.setId(procInstId);
         processAssignEntity.setProcInstId(procInstId);
         processAssignEntity.setSequenceNo(sequenceNo);
         processAssignEntity.setLoadTime(String.valueOf(System.currentTimeMillis()));
         setOpIndexUrl(processAssignEntity);
         save(processAssignEntity);
     }

     /**
      * findProcFormOpIndexUrl接口，返回url
      *
      * @param processAssignEntity
      */
     private void setOpIndexUrl(ProcessAssignEntity processAssignEntity) throws Exception {
         if (StringUtils.isBlank(processAssignEntity.getSequenceNo())) {
             return;
         }
         Map<String, Object> params = new HashMap<String, Object>();
         JSONObject bodyJSON = JSONObject.create();
         JSONObject paramObject = JSONObject.create();
         paramObject.put("sequenceNo", processAssignEntity.getSequenceNo());
         paramObject.put("authFlag", "0");
         bodyJSON.put("paramObject", paramObject);
         bodyJSON.put("formSysId", StringUtils.EMPTY);
         bodyJSON.put("sysId", SYS_ID);
         bodyJSON.put("userId", processAssignEntity.getDrafId());
         LogKit.info("cdia-CustomSubmitJob-setOpIndexUrl-params:{}", params);
         String res = postJSON(CdiaConfig.getInstance().getProcFormOpIndexUrl(), bodyJSON);
         LogKit.info("cdia-CustomSubmitJob-setOpIndexUrl-res:{}", res);
         String resultData = (new JSONObject(res)).getString("resultData");
         if (StringUtils.isNotBlank(resultData)) {
             processAssignEntity.setOpIndexUrl(resultData);
         }
     }

     /**
      * OA创建流程接口-国产化
      *
      * @param paramsJSON
      * @return
      */
     private JSONObject saveProcessBody(JSONObject paramsJSON) {
         JSONObject requestBody = JSONObject.create();
         JSONObject paramObject = new JSONObject().put("deptId", paramsJSON.getString("deptId"))
                 .put("docTitle", paramsJSON.getString("docTitle"))
                 .put("formValue", JSONObject.EMPTY)
                 .put("procDefId", paramsJSON.getString("procDefId"))
                 .put("isWait", "0");
         requestBody.put("paramObject", paramObject)
                 .put("sysId", paramsJSON.getString("systemCode"))
                 .put("formSysId", paramsJSON.getString("equipmentType"))
                 .put("userId", paramsJSON.getString("userId"));
         return requestBody;
     }
 }