 /*
  * Copyright (C), 2018-2022
  * Project: starter
  * FileName: CustomTotalSubmitJob
  * Author:   
  * Date:     2022/4/7 23:39
  */
 package com.fr.plugin.cdia.submit.job;

 import com.fanruan.api.log.LogKit;
 import com.fanruan.api.net.http.HttpKit;
 import com.fr.base.Parameter;
 import com.fr.data.JobValue;
 import com.fr.data.TotalSubmitJob;
 import com.fr.general.ComparatorUtils;
 import com.fr.general.DefaultValues;
 import com.fr.general.GeneralContext;
 import com.fr.json.JSONObject;
 import com.fr.plugin.cdia.ProcessDBAccess;
 import com.fr.plugin.cdia.bean.ProcessAssignBean;
 import com.fr.plugin.cdia.config.CdiaConfig;
 import com.fr.plugin.cdia.dao.ProcessAssignDao;
 import com.fr.plugin.cdia.entity.ProcessAssignEntity;
 import com.fr.script.Calculator;
 import com.fr.stable.StableUtils;
 import com.fr.stable.StringUtils;
 import com.fr.stable.db.action.DBAction;
 import com.fr.stable.db.dao.DAOContext;
 import com.fr.third.org.apache.http.entity.StringEntity;

 import java.util.*;

 /**
  * <Function Description><br>
  * <CustomTotalSubmitJob>
  *
  * @author 
  * @since 1.0.0
  */
 public class CustomTotalSubmitJob extends TotalSubmitJob {
     public static final String SYS_ID = "OA";
     private static final long serialVersionUID = 8626781409712334459L;
     private static final HashSet<String> paramsSet = new HashSet<String>() {{
         add("procDefId");
         add("docTitle");
         add("userId");
         add("deptId");
         add("systemCode");
         add("equipmentType");
     }};

     /**
      * POST提交JSON
      *
      * @param url
      * @param params
      * @return
      * @throws Exception
      */
     public static String postJSON(String url, JSONObject params) throws Exception {
         Map<String, String> headers = new HashMap<>();
         headers.put("Content-Type", "application/json");
         StringEntity stringEntity = new StringEntity(params.encode(), "UTF-8");
         return HttpKit.executeAndParse(com.fanruan.api.net.http.rs.HttpRequest.custom()
                 .url(url).post(stringEntity).headers(headers).build());
     }

     @Override
     public void doJob(Calculator calculator) throws Exception {
         super.doJob(calculator);
     }

     @Override
     public String getJobType() {
         return "CustomTotalJob";
     }

     @Override
     protected void doTotalJob(Data data, Calculator calculator) throws Exception {
         List<ProcessAssignBean> processAssignBeans = new ArrayList<>();
         ProcessAssignBean processAssignBean;
         ProcessAssignEntity processAssignEntity;
         JSONObject paramsJSON;
         boolean isRecordModify;
         for (int row = 0; row < data.getRowCount(); row++) {
             processAssignBean = new ProcessAssignBean();
             processAssignEntity = new ProcessAssignEntity();
             paramsJSON = JSONObject.create();
             isRecordModify = false;
             for (int col = 0; col < data.getColumnCount(); col++) {
                 Object cellValue = data.getValueAt(row, col);
                 if (!(cellValue instanceof JobValue)) {
                     continue;
                 }
                 JobValue ce = (JobValue) cellValue;
                 if (!isRecordModify && isStateStrict(ce)) {
                     isRecordModify = true;
                 }
                 if (paramsSet.contains(data.getColumnName(col))) {
                     paramsJSON.put(data.getColumnName(col), valueToString(ce.getValue()));
                 }
                 processAssignEntity.setEntity(data.getColumnName(col), valueToString(ce.getValue()));
             }
             if (isRecordModify) {
                 processAssignBean.setParamsJSON(paramsJSON);
                 processAssignBean.setProcessAssignEntity(processAssignEntity);
                 processAssignBeans.add(processAssignBean);
             }
         }
         if (processAssignBeans.isEmpty()) {
             return;
         }
         JSONObject procInstanceJo = JSONObject.EMPTY;
         for (ProcessAssignBean processAssign : processAssignBeans) {
             if (processAssign.getParamsJSON().isEmpty()) {
                 continue;
             }
             if (ComparatorUtils.equals(procInstanceJo, JSONObject.EMPTY)) {
                 procInstanceJo = getProcInstance(processAssign.getParamsJSON());
             }
             operation(procInstanceJo, processAssign.getProcessAssignEntity());
         }
     }

     private JSONObject getProcInstance(JSONObject paramsJSON) throws Exception {
         JSONObject createProcessBody = saveProcessBody(paramsJSON);
         LogKit.info("cdia-CustomTotalSubmitJob-getProcInstance-createProcessBody:{}", createProcessBody.encode());
         String res = postJSON(CdiaConfig.getInstance().getCreateProcessUrl(), createProcessBody);
         LogKit.info("cdia-CustomTotalSubmitJob-getProcInstance-response:{}", res);
         return new JSONObject(res).getJSONObject("resultData").getJSONObject("procInstance");
     }

     /**
      * jobValue to String
      *
      * @return
      */
     private String valueToString(Object value) {
         String result;
         if (value instanceof Date) {
             DefaultValues defaultValues = GeneralContext.getDefaultValues();
             result = defaultValues.getDateTimeFormat().format(value);
         } else if (value instanceof Parameter) {
             result = ((Parameter) value).valueToString();
         } else if (value instanceof Double) {
             result = StableUtils.doubleToString((Double) value);
         } else {
             result = String.valueOf(value);
         }
         return result;
     }

     /**
      * 填报属性状态判断
      *
      * @param jobValue
      * @return
      */
     private Boolean isStateStrict(JobValue jobValue) {
         if ((jobValue.getState().checkChanged() && !jobValue.getState().checkDeleted())) {
             return true;
         } else if (jobValue.getState().checkInsert()) {
             return true;
         } else if (jobValue.getState().checkDeleted()) {
             return true;
         } else if (jobValue.getState().checkDefault()) {
             return true;
         }
         return false;
     }

     /**
      * 提交属性处理操作
      * 1.访问创建流程接口，获得流程ID及其他信息
      * 2.流程表单保存
      * 3.打开浏览器窗口
      *
      * @param procInstanceJo
      * @param processAssignEntity
      * @throws Exception
      */
     private void operation(JSONObject procInstanceJo, ProcessAssignEntity processAssignEntity) throws Exception {
         String procInstId = procInstanceJo.getString("procInstId");
         String sequenceNo = procInstanceJo.getString("sequenceNo");
         if (StringUtils.isEmpty(procInstId) || ComparatorUtils.equals(procInstId, "{}")
                 || StringUtils.isEmpty(sequenceNo) || ComparatorUtils.equals(sequenceNo, "{}")
         ) {
             return;
         }
         // 保存数据库
         processAssignEntity.setId(UUID.randomUUID().toString());
         processAssignEntity.setProcInstId(procInstId);
         processAssignEntity.setSequenceNo(sequenceNo);
         processAssignEntity.setLoadTime(String.valueOf(System.currentTimeMillis()));
         setOpIndexUrl(processAssignEntity);
         save(processAssignEntity);
     }

     /**
      * findProcFormOpIndexUrl接口，返回url
      *
      * @param processAssignEntity
      */
     private void setOpIndexUrl(ProcessAssignEntity processAssignEntity) throws Exception {
         if (StringUtils.isBlank(processAssignEntity.getSequenceNo())) {
             return;
         }
         JSONObject bodyJSON = JSONObject.create();
         JSONObject paramObject = JSONObject.create();
         paramObject.put("sequenceNo", processAssignEntity.getSequenceNo());
         paramObject.put("authFlag", "0");
         bodyJSON.put("paramObject", paramObject);
         bodyJSON.put("formSysId", StringUtils.EMPTY);
         bodyJSON.put("sysId", SYS_ID);
         bodyJSON.put("userId", processAssignEntity.getDrafId());
         LogKit.info("cdia-CustomTotalSubmitJob-setOpIndexUrl-bodyJSON:{}", bodyJSON.encode());
         String res = postJSON(CdiaConfig.getInstance().getProcFormOpIndexUrl(), bodyJSON);
         LogKit.info("cdia-CustomTotalSubmitJob-setOpIndexUrl-res:{}", res);
         String resultData = (new JSONObject(res)).getString("resultData");
         if (StringUtils.isNotBlank(resultData)) {
             processAssignEntity.setOpIndexUrl(resultData);
         }
     }

     /**
      * OA创建流程接口-国产化
      *
      * @param paramsJSON
      * @return
      */
     private JSONObject saveProcessBody(JSONObject paramsJSON) {
         JSONObject requestBody = JSONObject.create();
         JSONObject paramObject = new JSONObject().put("deptId", paramsJSON.getString("deptId"))
                 .put("docTitle", paramsJSON.getString("docTitle"))
                 .put("formValue", JSONObject.EMPTY)
                 .put("procDefId", paramsJSON.getString("procDefId"))
                 .put("isWait", "0");
         requestBody.put("paramObject", paramObject)
                 .put("sysId", paramsJSON.getString("systemCode"))
                 .put("formSysId", paramsJSON.getString("equipmentType"))
                 .put("userId", paramsJSON.getString("userId"));
         return requestBody;
     }

     /**
      * 保存
      *
      * @param processAssignEntity
      * @throws Exception
      */
     private void save(final ProcessAssignEntity processAssignEntity) throws Exception {
         ProcessDBAccess.getAccessor().runDMLAction(new DBAction<ProcessAssignEntity>() {
             @Override
             public ProcessAssignEntity run(DAOContext daoContext) throws Exception {
                 (daoContext.getDAO(ProcessAssignDao.class)).addOrUpdate(processAssignEntity);
                 return null;
             }
         });
     }
 }