 /*
  * Copyright (C), 2018-2020
  * Project: starter
  * FileName: DataSubmitConfigPane
  * Author:   
  * Date:     2020/8/31 21:59
  */
 package com.fr.plugin.cdia.submit.ui;

 import com.fr.data.AbstractClassJob;
 import com.fr.data.ClassSubmitJob;
 import com.fr.data.SubmitJob;
 import com.fr.design.beans.BasicBeanPane;
 import com.fr.design.designer.properties.items.Item;
 import com.fr.design.gui.frpane.ObjectProperiesPane;
 import com.fr.design.gui.icombobox.UIComboBox;
 import com.fr.design.gui.itextarea.UITextArea;
 import com.fr.design.i18n.Toolkit;
 import com.fr.design.layout.FRGUIPaneFactory;
 import com.fr.design.utils.gui.GUICoreUtils;

 import javax.swing.*;
 import java.awt.*;
 import java.util.List;

 /**
  * <Function Description><br>
  * <数据提交配置DataSubmitConfigPane>
  *
  * @author 
  * @since 1.0.0
  */
 public class DataSubmitConfigPane extends BasicBeanPane {
     protected UIComboBox classNameComboBox;
     protected ObjectProperiesPane objectProperiesPane;

     public DataSubmitConfigPane() {
         this.setLayout(FRGUIPaneFactory.createBorderLayout());
         JPanel classPathPane = FRGUIPaneFactory.createBoxFlowInnerContainer_S_Pane();
         this.classNameComboBox = initUIComboBox(DataSubmitItems.LIST_ITEMS);
         classPathPane.add(this.classNameComboBox);
         classPathPane.setBorder(GUICoreUtils.createTitledBorder(Toolkit.i18nText("Fine-Design_Basic_Class_Name"), null));
         this.add(classPathPane, "North");
         this.objectProperiesPane = new ObjectProperiesPane();
         this.objectProperiesPane.setBorder(GUICoreUtils.createTitledBorder(Toolkit.i18nText("Fine-Design_Basic_Property"), null));
         this.add(this.objectProperiesPane, "Center");
         UITextArea description = new UITextArea(2, 1);
         description.setText(Toolkit.i18nText("Fine-Design_Basic_Extend_Class", new Object[]{"com.fr.data.AbstractSubmitTask"}));
         JPanel descriptionPane = FRGUIPaneFactory.createBorderLayout_S_Pane();
         descriptionPane.add(description);
         descriptionPane.setBorder(GUICoreUtils.createTitledBorder(Toolkit.i18nText("Fine-Design_Basic_Custom_Job_Description"), null));
         this.add(descriptionPane, "South");
         this.checkAddButtonEnable();
     }

     public int getLengthOfTextField() {
         return 25;
     }

     protected String title4PopupWindow() {
         return "DataSubmitJob";
     }

     protected Component getWindowAncestor() {
         return this;
     }

     public void populateBean(Object obj) {
         if (obj instanceof AbstractClassJob) {
             AbstractClassJob job = (AbstractClassJob) obj;
             this.classNameComboBox.setSelectedIndex(DataSubmitItems.classNameList.indexOf(job.getClassName()));
             this.objectProperiesPane.populateBean(job.getPropertyMap());
             this.checkAddButtonEnable();
         }
     }

     public SubmitJob updateBean() {
         ClassSubmitJob submitJob = new ClassSubmitJob(DataSubmitItems.classNameList.get(this.classNameComboBox.getSelectedIndex()));
         submitJob.setPropertyMap(this.objectProperiesPane.updateBean());
         this.checkAddButtonEnable();
         return submitJob;
     }

     public void checkAddButtonEnable() {
         this.objectProperiesPane.enableAddButton(true);
     }

     public void reset() {
         this.classNameComboBox.setSelectedIndex(0);
         this.checkAddButtonEnable();
     }

     /**
      * 初始化类名下拉框
      *
      * @param items
      * @return
      */
     private UIComboBox initUIComboBox(List<Item> items) {
         DefaultComboBoxModel model = new DefaultComboBoxModel();
         for (Item item : items) {
             model.addElement(item);
         }
         return new UIComboBox(model);
     }
 }